"""
Generic order snippets chain.

: `content_flows_field`
    The field within the document that contains the content flows (defaults to
    'flows').

"""

import flask
import json
from manhattan.chains import Chain, ChainMgr
from manhattan.content.views import factories
from manhattan.manage.views import utils as manage_utils
from manhattan.manage.views import factories as manage_factories

__all__ = ['order_snippets_chains']


# Define the chains
order_snippets_chains = ChainMgr()

# POST
order_snippets_chains['post'] = Chain([
    'config',
    'authenticate',
    'get_document',
    'get_flow',
    'order_snippets',
    'update_flow',
    'render_json'
])


# Define the links
order_snippets_chains.set_link(
    manage_factories.config(content_flows_field='flows')
)
order_snippets_chains.set_link(manage_factories.authenticate())
order_snippets_chains.set_link(manage_factories.get_document())
order_snippets_chains.set_link(factories.get_flow())
order_snippets_chains.set_link(factories.update_flow())

@order_snippets_chains.link
def order_snippets(state):
    """Order the snippets in a flow"""

    # Get the snippet Ids to use to set the order
    snippet_ids = flask.request.values.get('snippets', None)
    if not snippet_ids:
        return manage_utils.json_fail('No `snippets` specified')

    # Attempt to convert the snippet Ids string to a list
    try:
        snippet_ids = json.loads(snippet_ids)
    except ValueError:
        return manage_utils.json_fail('`snippets` is not valid JSON')

    # Build a look up table for snippets within the flow
    snippets = {s.id: s for s in state.flow}

    # Order the snippets
    ordered_flow = []
    for snippet_id in snippet_ids:
        if snippet_id in snippets:
            ordered_flow.append(snippets[snippet_id])

    if len(ordered_flow) != len(state.original_flow):
        return manage_utils.json_fail('Snippet Ids do not match the flow')

    state.flow = ordered_flow

@order_snippets_chains.link
def render_json(state):
    """Render a JSON response for the successful ordering of the snippets"""
    return manage_utils.json_success()



"""
Generic update contents for documents with a field containing content regions.

In some cases a fixed layout is preferable to the more flexible flow layout,
or you may wish to allow users to update more than one field within a document
via the in-page editor. This generic view is designed to provide a basis for
such cases (it's also possible to combine both flow and region based update
views).

Image syncing is only supported against the `content_regions_field`, assets
are stored within the regions field under the special `__assets__` key.

: `content_flows_field`
    The field within the document that contains the content flows (defaults to
    'flows').

"""

import json
import re

from bs4 import BeautifulSoup
import flask
from manhattan.assets import Asset
from manhattan.assets.transforms.base import BaseTransform
from manhattan.assets.transforms.images import Fit
from manhattan.chains import Chain, ChainMgr
from manhattan.content.snippets import GlobalRegion
from manhattan.content.views import factories
from manhattan.manage.views import utils as manage_utils
from manhattan.manage.views import factories as manage_factories
from mongoframes import Q

__all__ = ['update_contents_chains']


# Define the chains
update_contents_chains = ChainMgr()

update_contents_chains['post'] = Chain([
    'config',
    'authenticate',
    'get_document',
    'get_contents',
    'get_regions',
    'sync_images',
    'update_document',
    'render_json'
])


# Define the links

update_contents_chains.set_link(
    manage_factories.config(content_regions_field='flows')
)
update_contents_chains.set_link(manage_factories.authenticate())
update_contents_chains.set_link(manage_factories.get_document())
update_contents_chains.set_link(factories.get_contents())

@update_contents_chains.link
def get_regions(state):
    """
    Get copies of the regions within the document we'll be updating.

    This link adds the `regions` key to the state, which contains a dictionary
    of existing regions contents.
    """

    # Ensure there's a table to get/set contents against
    assert state.content_regions_field, 'No `content_regions_field` configured'
    regions = document.get(state.content_regions_field) or {}
    setattr(document, state.content_regions_field, regions)
    state.regions = regions.copy()

    # Ensure there's a base region and assets table for each of the content
    # updates.
    for region_name, content in state.contents.items():
        if region_name not in state.regions:
            state.regions[region_name] = {}
        if '__assets__' not in state.regions[region_name]:
            state.regions[region_name]['__assets__'] = {}

@update_contents_chains.link
def sync_images(state):
    """
    Temporary and permenant images inserted into the contents must be
    synchronized, by which we mean that temporary assets are converted to
    permenant assets and the content variation of the image is resized to
    match the width/height.

    This link adds the key `assets` to state which contains a dictionary of
    assets values for global regions, these are then used to update the global
    regions in the `update_contents` step.
    """
    asset_mgr = flask.current_app.asset_mgr

    # Get the variation name for content images
    config = flask.current_app.config
    variation_name = config.get('CONTENT_VARIATION_NAME', 'image')

    # Search the content for image tags with asset keys
    image_tags = []
    for region_name, content in state.contents.items():
        soup = BeautifulSoup(content, 'lxml')
        keyed_tags = soup.find_all(**{'data-mh-asset-key': True})
        for tag in keyed_tags:
            if tag.name == 'img':
                image_tags.append((region_name, tag))
            else:
                # Cater for image fixtures
                image_tag = tag.find('img')
                image_tag['data-mh-asset-key'] = tag['data-mh-asset-key']
                image_tags.append((region_name, image_tag))

    # Store and resize any images within the contents

    # Build a map of existing images for the region
    asset_map = {a[0]: a[1] for a in region.get('__assets__', [])}

    for image_tag in image_tags:
        region_name = image_tag[0]
        region = state.regions[region_name]
        tag = image_tag[1]

        # Extract the asset key, width and height from the image
        asset_key = tag['data-mh-asset-key']
        url = tag['src']
        width = tag.attrs.get('width', '')

        # If there's no associated asset key then ignore the image
        if not asset_key:
            continue

        # If a valid width is specified create a resize transform
        resize_transform = None
        if width.isdigit() and int(width) > 0:
            # Create a resize transform that will be used to set the size of the
            # image within the content.
            width = int(width)

            # HACK: We set the height to an arbitrary high value so we can
            # simplify the problem to just width.
            #
            # ~ Anthony Blackshaw <ant@getme.co.uk>, 31 August 2017
            #
            resize_transform = Fit(width, 99999)

        else:
            # No valid width so set the width to None
            width = None

        # If the image asset is temporary then we need to store a permenant
        # copy of it.
        asset = None

        if asset_mgr.get_temporary_by_key(asset_key):
            # New image
            asset = asset_mgr.get_temporary_by_key(asset_key)

            # Store the asset permenantly
            flask.current_app.asset_mgr.store(asset)

        elif asset_key in asset_map:
            # Existing image
            asset = Asset(assets_map[asset_key])

            # If this is a permenant asset check if we need to resize it
            variation = asset.variations.get(variation_name)

            if variation:
                size = variation['core_meta']['image']['size']
                if width == size[0]:
                    continue

        else:
            # Asset not found so continue
            continue

        # Create to match the base transform
        transforms = [
            BaseTransform.from_json_type(t) for t in asset.base_transforms]

        # If there's a width specified then a resize will be required
        if width:
            transforms += [resize_transform]

        asset_mgr.generate_variations(asset, {variation_name: transforms})

        if width:
            asset.variations[variation_name].local_transforms = [
                resize_transform.to_json_type()
            ]

        # Store the updated asset against the snippet's contents
        asset_json = asset.to_json_type()
        asset_map[asset.key] = asset_json

        # Replace the existing image src within the content with the new one
        new_url = asset.variations[variation_name].url
        state.contents[region_name] = \
                state.contents[region_name].replace(url, new_url)

    region['__assets__'] = [[k, v] for k, v in asset_map.items()]

@update_contents_chains.link
def update_document(state):
    """Update documents contents"""

    # Get the document we're updating the contents of
    document = state[state.manage_config.var_name]

    # Update the regions
    for region_name, content in state.contents.items():
        state.regions[region_name] = content

    # Update the document
    state.global_regions[region_name].logged_update(
        state.manage_user,
        {state.content_regions_field: state.regions}
    )

@update_contents_chains.link
def render_json(state):
    """Render a JSON response for the successful saving of content"""
    return manage_utils.json_success()



"""
Generic get snippets chain.

: `content_flows_field`
    The field within the document that contains the content flows (defaults to
    'flows').

"""

import flask
from manhattan.chains import Chain, ChainMgr
from manhattan.content.snippets import Snippet
from manhattan.content.views import factories
from manhattan.manage.views import utils as manage_utils
from manhattan.manage.views import factories as manage_factories

__all__ = ['snippets_chains']


# Define the chains
snippets_chains = ChainMgr()

# GET
snippets_chains['get'] = Chain([
    'config',
    'authenticate',
    'get_document',
    'get_flow',
    'render_json'
    ])


# Define the links
snippets_chains.set_link(manage_factories.config(content_flows_field='flows'))
snippets_chains.set_link(manage_factories.authenticate())
snippets_chains.set_link(manage_factories.get_document())
snippets_chains.set_link(factories.get_flow())

@snippets_chains.link
def render_json(state):
    """Render and return the snippets in JSON format"""

    # Build the payload
    payload = {'snippets': [s.to_json_type() for s in state.flow]}

    return manage_utils.json_success(payload)



"""
Generic change snippet scope chain.

: `content_flows_field`
    The field within the document that contains the content flows (defaults to
    'flows').

"""

import flask
from manhattan.chains import Chain, ChainMgr
from manhattan.content.snippets import GlobalSnippet
from manhattan.content.views import factories
from manhattan.manage.views import utils as manage_utils
from manhattan.manage.views import factories as manage_factories
from mongoframes import Q

__all__ = ['change_snippet_scope_chains']


# Define the chains
change_snippet_scope_chains = ChainMgr()

# POST
change_snippet_scope_chains['post'] = Chain([
    'config',
    'authenticate',
    'get_document',
    'get_flow',
    'get_snippet',
    'change_snippet_scope',
    'update_flow',
    'render_json'
])


# Define the links
change_snippet_scope_chains.set_link(
    manage_factories.config(content_flows_field='flows')
)
change_snippet_scope_chains.set_link(manage_factories.authenticate())
change_snippet_scope_chains.set_link(manage_factories.get_document())
change_snippet_scope_chains.set_link(factories.get_flow())
change_snippet_scope_chains.set_link(factories.get_snippet())
change_snippet_scope_chains.set_link(factories.update_flow())

@change_snippet_scope_chains.link
def change_snippet_scope(state):
    """Change the scope of a snippet"""

    # Get the scope we're changing to
    scope = flask.request.values.get('scope', None)
    if scope not in ['global', 'local']:
        return manage_utils.json_fail('`scope` must be local or global')

    # Check the scope needs changing
    if state.snippet.scope == scope:
        return

    # Change the scope of the snippet
    if scope == 'local':
        # Global > Local

        # Store settings and content slocally
        settings = state.snippet.settings
        contents = state.snippet.contents
        state.snippet.scope = 'local'
        state.snippet.local_settings = settings
        state.snippet.local_contents = contents

        # Remove the global Id from the snippet
        del state.snippet._document['global_id']

    else:
        # Local > Global

        # Get the label to use for global snippet
        label = flask.request.values.get('label', None).strip()
        if not label:
            return manage_utils.json_fail(
                'No `label` specified',
                {'label': 'This field is required'}
                )

        # Check the label is unique
        if GlobalSnippet.count(Q.label == label):
            return manage_utils.json_fail(
                'Not a valid label',
                {'label': 'Labels must be unqiue'}
                )

        # Create the global snippet
        global_snippet = GlobalSnippet(
            type=state.snippet.type,
            visible=True,
            settings=state.snippet.local_settings,
            contents=state.snippet.contents,
            label=label
        )
        global_snippet.insert()

        # Update the snippet so it references the new global snippet
        state.snippet.scope = 'global'
        state.snippet.local_settings = {}
        state.snippet.local_contents = {}
        state.snippet.global_id = global_snippet._id

@change_snippet_scope_chains.link
def render_json(state):
    """
    Render a JSON response for the successful change of the snippet's scope.
    """
    return manage_utils.json_success()



"""
Generic update global contents asset chain.

This view updates contents with the `GlobalRegion` collection.
"""

import json
import re

from bs4 import BeautifulSoup
import flask
from manhattan.assets import Asset
from manhattan.assets.transforms.base import BaseTransform
from manhattan.assets.transforms.images import Fit
from manhattan.chains import Chain, ChainMgr
from manhattan.content.snippets import GlobalRegion
from manhattan.content.views import factories
from manhattan.manage.views import utils as manage_utils
from manhattan.manage.views import factories as manage_factories
from mongoframes import Q

__all__ = ['update_global_contents_chains']


# Define the chains
update_global_contents_chains = ChainMgr()

update_global_contents_chains['post'] = Chain([
    'config',
    'authenticate',
    'get_contents',
    'get_global_regions',
    'sync_images',
    'update_contents',
    'render_json'
])


# Define the links

update_global_contents_chains.set_link(manage_factories.config())
update_global_contents_chains.set_link(manage_factories.authenticate())
update_global_contents_chains.set_link(factories.get_contents())

@update_global_contents_chains.link
def get_global_regions(state):
    """
    Get the global regions that will be update based on the contents.

    This link adds the `global_regions` key to the state, which contains a
    dictionary of global regions to apply the update to, e.g
    `{region_name: instance}`.

    NOTE: If the contents contains a region name that doesn't exist a new
    region will be created.
    """

    # Build the table of global regions
    global_regions = {}
    for region_name, content in state.contents.items():

        # Attempt to find the named global region
        global_region = GlobalRegion.one(Q.name == region_name)

        # If there's no existing region with that name then add one
        if not global_region:
            global_region = GlobalRegion(name=region_name)
            global_region.logged_insert(state.manage_user)

        global_regions[region_name] = global_region

    # Add the global regions to the state
    state.global_regions = global_regions

@update_global_contents_chains.link
def sync_images(state):
    """
    Temporary and permenant images inserted into the contents must be
    synchronized, by which we mean that temporary assets are converted to
    permenant assets and the content variation of the image is resized to
    match the width/height.

    This link adds the key `assets` to state which contains a dictionary of
    assets values for global regions, these are then used to update the global
    regions in the `update_contents` step.
    """
    asset_mgr = flask.current_app.asset_mgr

    # Take a copy of the assets for each global region we're updating
    state.assets = {}
    for region_name, content in state.contents.items():
        global_region = state.global_regions[region_name]
        state.assets[region_name] = global_region.assets.copy()

    # Get the variation name for content images
    config = flask.current_app.config
    variation_name = config.get('CONTENT_VARIATION_NAME', 'image')

    # Search the content for image tags with asset keys
    image_tags = []
    for region_name, content in state.contents.items():
        soup = BeautifulSoup(content, 'lxml')
        keyed_tags = soup.find_all(**{'data-mh-asset-key': True})
        for tag in keyed_tags:
            if tag.name == 'img':
                image_tags.append((region_name, tag))
            else:
                # Cater for image fixtures
                image_tag = tag.find('img')
                image_tag['data-mh-asset-key'] = tag['data-mh-asset-key']
                image_tags.append((region_name, image_tag))

    # Store and resize any images within the contents
    for image_tag in image_tags:
        region_name = image_tag[0]
        global_region = state.global_regions[region_name]
        tag = image_tag[1]

        # Extract the asset key, width and height from the image
        asset_key = tag['data-mh-asset-key']
        url = tag['src']
        width = tag.attrs.get('width', '')

        # If there's no associated asset key then ignore the image
        if not asset_key:
            continue

        # If a valid width is specified create a resize transform
        resize_transform = None
        if width.isdigit() and int(width) > 0:
            # Create a resize transform that will be used to set the size of the
            # image within the content.
            width = int(width)

            # HACK: We set the height to an arbitrary high value so we can
            # simplify the problem to just width.
            #
            # ~ Anthony Blackshaw <ant@getme.co.uk>, 31 August 2017
            #
            resize_transform = Fit(width, 99999)

        else:
            # No valid width so set the width to None
            width = None

        # If the image asset is temporary then we need to store a permenant
        # copy of it.
        asset = None
        if asset_mgr.get_temporary_by_key(asset_key):
            # New image
            asset = asset_mgr.get_temporary_by_key(asset_key)

            # Store the asset permenantly
            flask.current_app.asset_mgr.store(asset)

        elif asset_key in global_region.assets:
            # Existing image
            asset = Asset(global_region.assets[asset_key])

            # If this is a permenant asset check if we need to resize it
            variation = asset.variations.get(variation_name)

            if variation:
                size = variation['core_meta']['image']['size']
                if width == size[0]:
                    continue

        else:
            # Asset not found so continue
            continue

        # Create to match the base transform
        transforms = [
            BaseTransform.from_json_type(t) for t in asset.base_transforms]

        # If there's a width specified then a resize will be required
        if width:
            transforms += [resize_transform]

        asset_mgr.generate_variations(asset, {variation_name: transforms})

        if width:
            asset.variations[variation_name].local_transforms = [
                resize_transform.to_json_type()
            ]

        # Store the updated asset against the snippet's contents
        asset_json = asset.to_json_type()
        state.assets[region_name][asset.key] = asset_json

        # Replace the existing image src within the content with the new one
        new_url = asset.variations[variation_name].url
        state.contents[region_name] = \
                state.contents[region_name].replace(url, new_url)

@update_global_contents_chains.link
def update_contents(state):
    """Update the contents of global regions within the contents"""
    for region_name, content in state.contents.items():
        state.global_regions[region_name].logged_update(
            state.manage_user,
            {'content': content, 'assets': state.assets[region_name]}
        )

@update_global_contents_chains.link
def render_json(state):
    """Render a JSON response for the successful saving of content"""
    return manage_utils.json_success()


"""
Generic update snippet settings chain.

: `content_flows_field`
    The field within the document that contains the content flows (defaults to
    'flows').

"""

import flask
from manhattan.chains import Chain, ChainMgr
from manhattan.content.views import factories
from manhattan.forms import BaseForm, fields, validators
from manhattan.manage.views import utils as manage_utils
from manhattan.manage.views import factories as manage_factories

__all__ = ['update_snippet_settings_chains']


# Define the chains
update_snippet_settings_chains = ChainMgr()

# GET
update_snippet_settings_chains['get'] = Chain([
    'config',
    'authenticate',
    'get_document',
    'get_flow',
    'get_snippet',
    'init_form',
    'convert_form_to_json_type',
    'render_get_json'
])

# POST
update_snippet_settings_chains['post'] = Chain([
    'config',
    'authenticate',
    'get_document',
    'get_flow',
    'get_snippet',
    'init_form',
    'validate',
    'build_form_data',
    'update_snippet_settings',
    'update_flow',
    'render_post_json'
])


# Define the links

update_snippet_settings_chains.set_link(
    manage_factories.config(content_flows_field='flows')
)
update_snippet_settings_chains.set_link(manage_factories.authenticate())
update_snippet_settings_chains.set_link(manage_factories.get_document())
update_snippet_settings_chains.set_link(factories.get_flow())
update_snippet_settings_chains.set_link(factories.get_snippet())
update_snippet_settings_chains.set_link(factories.update_flow())

@update_snippet_settings_chains.link
def init_form(state):
    """
    Initialize a form for the settings of snippet being updated, the form is
    aquired from the snippet's snippet type class.

    This link adds a `form` key to the the state containing the initialized
    form.
    """

    # Check the snippet's type has a settings form
    form = None
    form_cls = state.snippet.type_cls.get_settings_form()
    if form_cls:

        # Initialize the form
        form_data = None
        if flask.request.method == 'POST':
            form_data = flask.request.form

        form = form_cls(form_data, **state.snippet.settings)

    # Add the form to the state
    state.form = form

@update_snippet_settings_chains.link
def validate(state):
    """
    Validate data POSTed to the view against `state.form`. If the form is valid
    the function will return `True` or `False` if not.
    """

    # Only attempt to validate if there's a form
    if not state.form:
        return

    # Validate the form
    if not state.form.validate():
        return manage_utils.json_fail(
            'Invalid form submission',
            state.form.errors
        )

@update_snippet_settings_chains.link
def build_form_data(state):
    """
    Generate the form data that will be used to update the snippet's settings.

    This link adds a `form_data` key to the the state.
    """

    state.form_data = {}
    if state.form:
        state.form_data = state.form.data

@update_snippet_settings_chains.link
def update_snippet_settings(state):
    """Update the snippet's settings"""

    if state.snippet.scope == 'local':
        # Apply changes to this snippet
        state.snippet.local_settings = state.form_data

    elif state.snippet.scope == 'global':
        # Apply changes to the global snippet this snippet references
        global_snippet = state.snippet.global_snippet

        # Update the global snippet
        global_snippet.logged_update(
            state.manage_user,
            {'settings': state.form_data}
        )

        # Clear global snippet cache from the snippet
        state.snippet._global_snippet_cache = None

@update_snippet_settings_chains.link
def convert_form_to_json_type(state):
    """
    Convert the settings form into to a list of fields represented as JSON
    safe data.

    ContentFlow supports boolean, select/options and text entry fields and so
    fields which can't be directly converted default to text entry.

    This link adds a `fields` key to the state containing the JSON type
    representation of the form fields.
    """

    # Field class to type map
    field_types = {
        fields.BooleanField: 'boolean',
        fields.DocumentSelectField: 'select',
        fields.SelectField: 'select'
    }

    # Serialize the form fields in to JSON safe data types
    form_fields = []
    if state.form:
        for field in state.form:

            # Get the initial value of the field
            value = field.default
            if field.name in state.snippet.settings:
                value = field.data

            # Build the base field data
            field_json = {
                'name': field.name,
                'label': field.label.text,
                'type': field_types.get(field.__class__, 'text'),
                'required': field.flags.required,
                'value': value
            }

            # Cater for different field types
            if isinstance(field, fields.SelectField):
                field_json['choices'] = field.choices

            form_fields.append(field_json)

    state.fields = form_fields

@update_snippet_settings_chains.link
def render_get_json(state):
    """Render a JSON representation of the snippets settings form"""

    return manage_utils.json_success({'fields': state.fields})

@update_snippet_settings_chains.link
def render_post_json(state):
    """
    Render a JSON response for the successful update of the snippet's
    settings.
    """

    # Get the document we're add the snippet to
    document = state[state.manage_config.var_name]

    # Render the snippet as HTML
    snippet_html = state.snippet.render(
        {state.manage_config.var_name: document}
    )

    return manage_utils.json_success({'html': snippet_html})


"""
Generic add global snippet chain.

: `content_flows_field`
    The field within the document that contains the content flows (defaults to
    'flows').

"""

import bson

import flask
from manhattan.chains import Chain, ChainMgr
from manhattan.content.snippets import GlobalSnippet, Snippet, SnippetType
from manhattan.content.views import factories
from manhattan.manage.views import utils as manage_utils
from manhattan.manage.views import factories as manage_factories
from mongoframes import And, Q

__all__ = ['add_global_snippet_chains']


# Define the chains
add_global_snippet_chains = ChainMgr()

# POST
add_global_snippet_chains['post'] = Chain([
    'config',
    'authenticate',
    'get_document',
    'get_flow',
    'get_global_snippet',
    'add_snippet',
    'update_flow',
    'render_json'
])


# Define the links
add_global_snippet_chains.set_link(
    manage_factories.config(content_flows_field='flows')
)
add_global_snippet_chains.set_link(manage_factories.authenticate())
add_global_snippet_chains.set_link(manage_factories.get_document())
add_global_snippet_chains.set_link(factories.get_flow())
add_global_snippet_chains.set_link(factories.update_flow())

@add_global_snippet_chains.link
def get_global_snippet(state):
    """
    Get the global snippet we'll be adding to the flow.

    This link adds `global_snippet` to the state.
    """

    # Get the snippet types supported by the flow
    snippet_types = SnippetType.get_snippet_types([state.flow_id])

    # Get the global snippet Id
    global_id = flask.request.values.get('global_snippet', None)
    if not global_id:
        return manage_utils.json_fail('No `global_snippet` specified')

    try:
        global_id = bson.objectid.ObjectId(global_id)
    except bson.errors.InvalidId:
        return manage_utils.json_fail('`global_snippet` is not a valid Id')

    # Get the global snippet
    global_snippet = GlobalSnippet.one(
        And(Q._id == global_id, Q.visible == True)
    )
    if not global_snippet:
        return manage_utils.json_fail('Global snippet not found')

    state.global_snippet = global_snippet

@add_global_snippet_chains.link
def add_snippet(state):
    """
    Add a global snippet to the named flow.

    This link adds `snippet` to the state.
    """

    # Create the new snippet
    snippet = Snippet(
        type=state.global_snippet.type,
        scope='global',
        global_id=state.global_snippet._id
    )

    # Add the snippet to the flow
    state.flow.append(snippet)

    # Add the snippet to the state
    state.snippet = snippet

@add_global_snippet_chains.link
def render_json(state):
    """Render the new snippet as HTML and return it in a JSON response"""

    # Get the document we're add the snippet to
    document = state[state.manage_config.var_name]

    # Render the snippet as HTML
    snippet_html = state.snippet.render(
        {state.manage_config.var_name: document}
    )

    return manage_utils.json_success({'html': snippet_html})
